/*
 * File:	ximasvi.cpp
 * Purpose:	Platform Independent SuperVozelj Image Class Loader and Writer
 * by Marco Savegnago IW3FQG - e-mail: iw3fqg@amsat.org
 * CxImage version 4.10 02/Jun/2002
 */

#include "ximasvi.h"

#include <io.h>

#if CXIMAGE_SUPPORT_SVI

#include "ximaiter.h" 

////////////////////////////////////////////////////////////////////////////////
bool CxImageSVI::Encode(CxFile * hFile)
{
	if (hFile==NULL) return false;

    // Write the DIB header and the pixels
	hFile->Write(pDib,GetSize(),1);

	return true;
}
////////////////////////////////////////////////////////////////////////////////
bool CxImageSVI::Decode(CxFile * hFile)
{
	if (hFile == NULL) return false;

	int		lastch, ch;
	DWORD	imgpos;
	
	try 
	{
		head.biSize               = sizeof(BITMAPINFOHEADER);
        head.biWidth              = (DWORD)info.nSVIXSize;
        head.biHeight             = (DWORD)info.nSVIYSize;
        head.biPlanes             = 1;
        head.biBitCount           = 8;
        head.biCompression        = 0;
        head.biSizeImage          = (DWORD)info.nSVIXSize * (DWORD)info.nSVIYSize;
        head.biXPelsPerMeter      = 0;
        head.biYPelsPerMeter      = 0;
        head.biClrUsed            = 0;
        head.biClrImportant       = 0;

		if (!Create(head.biWidth,head.biHeight,head.biBitCount))
			throw "Can't allocate memory";

		info.xDPI = (long) floor(head.biXPelsPerMeter * 254.0 / 10000.0 + 0.5);
		info.yDPI = (long) floor(head.biYPelsPerMeter * 254.0 / 10000.0 + 0.5);

		SetGrayPalette();

		lastch	= ch =	0;
		imgpos	=	0;
		
		hFile->Seek(0,SEEK_SET);

		if(info.bSVIRemoveGarbage){
			while(1){
   				ch	=	hFile->GetC();
				if(ch == '\n' && lastch	== '\r'){
					//	found a cr + lf
					imgpos	=	hFile->Tell();
				}
				if(hFile->Eof()) break;
				lastch	=	ch;
			}

			hFile->Seek(0,SEEK_SET);
			
			if(imgpos<info.dwSVIGarbageLen)	hFile->Seek(imgpos, SEEK_SET);
		}

		hFile->Read(info.pImage, head.biSizeImage,1); // read in the pixels

		Flip();

	} 
	catch (char *message) 
	{
		strncpy(info.szLastError,message,255);
		return FALSE;
	}
    
	return true;
}
////////////////////////////////////////////////////////////////////////////////
#endif 	// CXIMAGE_SUPPORT_SVI
